'''
Created on Nov 4, 2015

@author: krgupta
'''

from ConfigParser import SafeConfigParser
from ConfigParser import NoSectionError
import os
import sys
#from __future__ import print_function

class helper(): 
    __parser = "null"
    __propertyfilename = "null"
    __initialized = False
    
    @staticmethod
    def getparser():
        return helper.__parser
    
    @staticmethod
    def getpropertyfile():
        return helper.__propertyfilename

    @staticmethod
    def setpropertyfile(propertyfilename):
        if (propertyfilename == 'null' or os.path.isfile(propertyfilename) == False):
            helper.__propertyfilename = 'null' 
        else:     
            helper.__propertyfilename = propertyfilename
        return

    @staticmethod
    def __classinitialized():
        return helper.__initialized
    
    @staticmethod
    def getproperty(propertyname):
        stringvalue = "null"

        if ('null' != helper.getpropertyfile()):
            if (False == helper.__classinitialized()):
                if ('null' == helper.getparser()):
                    try:
                        helper.__parser = SafeConfigParser({"http":"","https":"","ftp":""})
                    except:
                        print ("Parser could not be initialized")

                if ('null' != helper.getparser()):
                    try:
                        helper.getparser().read(helper.__propertyfilename)
                        helper.__initialized = True
                    except:
                        print ("Unable to load the property file")

        if (True == helper.__classinitialized()):
            try:
                stringvalue = helper.getparser().get("properties", propertyname)
            except:
                sys.stderr.write("%s not found" %propertyname )
                
        if ( "null" == stringvalue):
            stringvalue = os.getenv(propertyname)               
        return stringvalue 


'''
Created on Jun 8, 2015

@author: egodolja
'''
import logging

class constants(object):
    """All the constants are defined here
    Define all your constants instead of using magic numbers in the
    code. 
    """
    
    '''Environments'''
    SANDBOX = 'https://apitest.authorize.net/xml/v1/request.api'
    PRODUCTION = 'https://api2.authorize.net/xml/v1/request.api'

    '''xml encoding'''
    xml_encoding = 'utf-8'
    
    '''xml headers'''
    headers = {'Content-Type' : 'application/xml', 'version' : '1.0', 'encoding' : xml_encoding}
    
    """
    Following constants are defined and used in the ARBSubscriptionStatusController
    Used to remove the "Status" element, that has been deprecated
    However, since the server response still contains it, we have to remove it
    before de-serialization
    """
    '''ARBGetSubscriptionStatus <Status> tag'''
    StatusStart = '<Status>'
    StatusEnd = '</Status>'
    
    '''response encoding'''
    response_encoding = 'ISO-8859-1'
    
    '''note section of subscription status response'''
    note = ' note="Status with a capital \'S\' is obsolete."'
    
    '''ns namespace 1'''
    nsNamespace1 = 'ns1:'
    
    '''ns namespace 2'''
    nsNamespace2 = ':ns1'
    
    '''default log file name'''
    defaultLogFileName = "anetsdk.log"
    
    '''default logging level'''
    #defaultLoggingLevel = logging.WARNING
    defaultLoggingLevel = logging.DEBUG
    
    '''default log format'''
    defaultlogformat = '%(asctime)s %(message)s'
    
    propertiesloggingfilename = "loggingfilename"
    
    propertiesexecutionlogginglevel = "executionlogginglevel"

    
'''eof'''



'''
Created on Nov 19, 2015

@author: krgupta
'''
import logging
from authorizenet.constants import constants
from authorizenet import apicontractsv1
from authorizenet import apicontrollersbase    
class ARBCancelSubscriptionController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(ARBCancelSubscriptionController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'ARBCancelSubscriptionRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.ARBCancelSubscriptionResponse() 
    
class ARBCreateSubscriptionController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(ARBCreateSubscriptionController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'ARBCreateSubscriptionRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.ARBCreateSubscriptionResponse()
     
class ARBGetSubscriptionController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(ARBGetSubscriptionController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'ARBGetSubscriptionRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.ARBGetSubscriptionResponse() 
class ARBGetSubscriptionListController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(ARBGetSubscriptionListController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'ARBGetSubscriptionListRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.ARBGetSubscriptionListResponse() 
    
class ARBGetSubscriptionStatusController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(ARBGetSubscriptionStatusController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'ARBGetSubscriptionStatusRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.ARBGetSubscriptionStatusResponse()
    
    def afterexecute(self):
        response = self._httpResponse
        if constants.note in response:
            response = response.replace(constants.note, '')

        if constants.StatusStart in response:
            start = response.index(constants.StatusStart)
            end = response.index(constants.StatusEnd)
            response = response.replace(response[start:end+9], '')

        self._httpResponse = response
        return
     
class ARBUpdateSubscriptionController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(ARBUpdateSubscriptionController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'ARBUpdateSubscriptionRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.ARBUpdateSubscriptionResponse() 
    
class authenticateTestController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(authenticateTestController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'authenticateTestRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.authenticateTestResponse() 
    
class createCustomerPaymentProfileController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(createCustomerPaymentProfileController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'createCustomerPaymentProfileRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.createCustomerPaymentProfileResponse() 
    
class createCustomerProfileController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(createCustomerProfileController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'createCustomerProfileRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.createCustomerProfileResponse() 
    
class createCustomerProfileFromTransactionController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(createCustomerProfileFromTransactionController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'createCustomerProfileFromTransactionRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.createCustomerProfileResponse() 
    
class createCustomerProfileTransactionController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(createCustomerProfileTransactionController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'createCustomerProfileTransactionRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.createCustomerProfileTransactionResponse() 
    
class createCustomerShippingAddressController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(createCustomerShippingAddressController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'createCustomerShippingAddressRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.createCustomerShippingAddressResponse() 
    
class createTransactionController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(createTransactionController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'createTransactionRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.createTransactionResponse() 
    
class decryptPaymentDataController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(decryptPaymentDataController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'decryptPaymentDataRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.decryptPaymentDataResponse() 
    
class deleteCustomerPaymentProfileController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(deleteCustomerPaymentProfileController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'deleteCustomerPaymentProfileRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.deleteCustomerPaymentProfileResponse() 
    
class deleteCustomerProfileController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(deleteCustomerProfileController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'deleteCustomerProfileRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.deleteCustomerProfileResponse() 
    
class deleteCustomerShippingAddressController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(deleteCustomerShippingAddressController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'deleteCustomerShippingAddressRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.deleteCustomerShippingAddressResponse() 
    
class ErrorController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(ErrorController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'ErrorRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.ErrorResponse() 
    
class getBatchStatisticsController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getBatchStatisticsController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getBatchStatisticsRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getBatchStatisticsResponse() 
    
class getCustomerPaymentProfileController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getCustomerPaymentProfileController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getCustomerPaymentProfileRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getCustomerPaymentProfileResponse() 
    
class getCustomerPaymentProfileListController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getCustomerPaymentProfileListController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getCustomerPaymentProfileListRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getCustomerPaymentProfileListResponse() 
    
class getCustomerProfileController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getCustomerProfileController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getCustomerProfileRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getCustomerProfileResponse() 
    
class getCustomerProfileIdsController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getCustomerProfileIdsController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getCustomerProfileIdsRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getCustomerProfileIdsResponse() 
    
class getCustomerShippingAddressController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getCustomerShippingAddressController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getCustomerShippingAddressRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getCustomerShippingAddressResponse() 
    
class getHostedProfilePageController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getHostedProfilePageController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getHostedProfilePageRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getHostedProfilePageResponse() 
    
class getSettledBatchListController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getSettledBatchListController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getSettledBatchListRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getSettledBatchListResponse() 
    
class getTransactionDetailsController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getTransactionDetailsController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getTransactionDetailsRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getTransactionDetailsResponse() 
    
class getTransactionListController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getTransactionListController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getTransactionListRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getTransactionListResponse() 
    
class getUnsettledTransactionListController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(getUnsettledTransactionListController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'getUnsettledTransactionListRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.getUnsettledTransactionListResponse() 
    
class isAliveController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(isAliveController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'isAliveRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.isAliveResponse() 
    
class logoutController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(logoutController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'logoutRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.logoutResponse() 
    
class mobileDeviceLoginController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(mobileDeviceLoginController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'mobileDeviceLoginRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.mobileDeviceLoginResponse() 
    
class mobileDeviceRegistrationController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(mobileDeviceRegistrationController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'mobileDeviceRegistrationRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.mobileDeviceRegistrationResponse() 
    
class sendCustomerTransactionReceiptController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(sendCustomerTransactionReceiptController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'sendCustomerTransactionReceiptRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.sendCustomerTransactionReceiptResponse() 
    
class updateCustomerPaymentProfileController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(updateCustomerPaymentProfileController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'updateCustomerPaymentProfileRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.updateCustomerPaymentProfileResponse() 
    
class updateCustomerProfileController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(updateCustomerProfileController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'updateCustomerProfileRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.updateCustomerProfileResponse() 
    
class updateCustomerShippingAddressController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(updateCustomerShippingAddressController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'updateCustomerShippingAddressRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.updateCustomerShippingAddressResponse()
     
class updateSplitTenderGroupController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(updateSplitTenderGroupController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'updateSplitTenderGroupRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.updateSplitTenderGroupResponse() 
    
class validateCustomerPaymentProfileController(apicontrollersbase.APIOperationBase):
    
    def __init__(self, apirequest):
        super(validateCustomerPaymentProfileController, self).__init__(apirequest)
        return 
    
    def validaterequest(self):
        logging.debug('performing custom validation..') 
        #validate required fields
        #if (self._request.xyz == "null"):
        #    raise ValueError('xyz is required')         
        return
    
    def getrequesttype(self):
        '''Returns request type''' 
        return 'validateCustomerPaymentProfileRequest'

    def getresponseclass(self):
        ''' Returns the response class '''
        return apicontractsv1.validateCustomerPaymentProfileResponse() 


'''
Created on Nov 1, 2015

@author: krgupta
'''
import abc
import logging
import pyxb
import sys
import xml.dom.minidom
from pip._vendor import requests
from _pyio import __metaclass__
from lxml import objectify

from authorizenet.constants import constants
from authorizenet import apicontractsv1
from authorizenet import utility
'''
from authorizenet.apicontractsv1 import merchantAuthenticationType
from authorizenet.apicontractsv1 import ANetApiRequest
from authorizenet.apicontractsv1 import ANetApiResponse
'''
class APIOperationBaseInterface(object):
    
    __metaclass__ = abc.ABCMeta
    
    @abc.abstractmethod
    def execute(self):
        '''
        Makes a http-post call. 
        Uses request xml and response class type to check that the response was of correct type
        '''
        pass

    @abc.abstractmethod
    def getresponseclass(self):
        ''' Returns the response class '''
        pass
    
    @abc.abstractmethod
    def getrequesttype(self):
        ''' Returns the request class '''
        pass
    
    @abc.abstractmethod
    def getresponse(self):
        ''' Returns the de-serialized response'''
        pass
    
    @abc.abstractmethod
    def getresultcode(self):
        ''' Returns the result code from the response '''
        pass
    
    @abc.abstractmethod
    def getmessagetype(self):
        ''' Returns the message type enum from the response '''
        pass

    @abc.abstractmethod
    def afterexecute(self):
        '''Returns the message received from binding after processing request'''
        pass

    @abc.abstractmethod
    def beforeexecute(self):
        '''TODO'''
        pass

class APIOperationBase(APIOperationBaseInterface):
    
    __metaclass__ = abc.ABCMeta 
    __initialized = False
    __merchantauthentication = "null"
    __environment = "null"
    
    @staticmethod
    def __classinitialized():
        return APIOperationBase.__initialized
    
    @abc.abstractmethod
    def validaterequest(self):
        return
    
    def validate(self):
        anetapirequest = self._getrequest()
        self.validateandsetmerchantauthentication()       
        self.validaterequest()
        
        return     
    
    def _getrequest(self): #protected method
        return self._request 
     
    def buildrequest(self):
        logging.debug('building request..')
        
        xmlRequest = self._request.toxml(encoding=constants.xml_encoding, element_name=self.getrequesttype())
        #remove namespaces that toxml() generates
        xmlRequest = xmlRequest.replace(constants.nsNamespace1, '')
        xmlRequest = xmlRequest.replace(constants.nsNamespace2, '')
        
        return xmlRequest
    
    def getprettyxmlrequest(self):
        xmlRequest = self.buildrequest()
        requestDom = xml.dom.minidom.parseString(xmlRequest)
        logging.debug('Request is: %s' % requestDom.toprettyxml())

        return requestDom
    
    def execute(self):
        
        self.endpoint = APIOperationBase.__environment
              
        logging.debug('Executing http post to url: %s', self.endpoint)
        
        self.beforeexecute()
        
        proxyDictionary = {'http' : utility.helper.getproperty("http"),
                           'https' : utility.helper.getproperty("https"),
                           'ftp' : utility.helper.getproperty("ftp")}
                           
        #requests is http request  
        try:
            xmlRequest = self.buildrequest()
            self._httpResponse = requests.post(self.endpoint, data=xmlRequest, headers=constants.headers, proxies=proxyDictionary)
        except Exception as httpException:
            logging.error( 'Error retrieving http response from: %s for request: %s', self.endpoint, self.getprettyxmlrequest())
            logging.error( 'Exception: %s, %s', type(httpException), httpException.args )


        if self._httpResponse:            
            self._httpResponse.encoding = constants.response_encoding
            self._httpResponse = self._httpResponse.text[3:] #strip BOM
            self.afterexecute()
            try:
                self._response = apicontractsv1.CreateFromDocument(self._httpResponse) 
                #objectify code  
                xmlResponse= self._response.toxml(encoding=constants.xml_encoding, element_name=self.getrequesttype()) 
                xmlResponse = xmlResponse.replace(constants.nsNamespace1, '')
                xmlResponse = xmlResponse.replace(constants.nsNamespace2, '') 
                self._mainObject = objectify.fromstring(xmlResponse)   
                 
            except Exception as objectifyexception:
                logging.error( 'Create Document Exception: %s, %s', type(objectifyexception), objectifyexception.args )
                pyxb.GlobalValidationConfig._setForBinding(False)
                self._response = apicontractsv1.CreateFromDocument(self._httpResponse)    
                #objectify code 
                xmlResponse= self._response.toxml(encoding=constants.xml_encoding, element_name=self.getrequesttype()) 
                xmlResponse = xmlResponse.replace(constants.nsNamespace1, '')
                xmlResponse = xmlResponse.replace(constants.nsNamespace2, '') 
                self._mainObject = objectify.fromstring(xmlResponse) 
            else:    
                #if type(self.getresponseclass()) == type(self._response):
                if type(self.getresponseclass()) != type(self._mainObject):
                    if self._response.messages.resultCode == "Error":
                        print "Response error"
                    domResponse = xml.dom.minidom.parseString(self._httpResponse)
                    logging.debug('Received response: %s' % domResponse.toprettyxml())
                else:
                    #Need to handle ErrorResponse  
                    logging.debug('Error retrieving response for request: %s' % self._request)
        else:
            print "Did not receive http response"
        return
    
    def getresponse(self):
        #return self._response #pyxb object
        return self._mainObject #objectify object
    
    def getresultcode(self):
        resultcode = 'null'
        if self._response:
            resultcode = self._response.resultCode
        return resultcode
    
    def getmessagetype(self):
        message = 'null'
        if self._response:
            message = self._response.message
        return message
    
    def afterexecute(self ):
        return 
    
    def beforeexecute(self):
        return 
    
    @staticmethod
    def getmerchantauthentication(self):
        return self.__merchantauthentication
    
    @staticmethod
    def setmerchantauthentication(merchantauthentication):
        APIOperationBase.__merchantauthentication = merchantauthentication
        return
    
    def validateandsetmerchantauthentication(self):
        anetapirequest = apicontractsv1.ANetApiRequest()
        if (anetapirequest.merchantAuthentication == "null"):
            if (self.getmerchantauthentication() != "null"):
                anetapirequest.merchantAuthentication = self.getmerchantauthentication()
            else:
                raise ValueError('Merchant Authentication can not be null')
        return
    
    @staticmethod
    def getenvironment(self):
        return APIOperationBase.__environment
        
    
    @staticmethod
    def setenvironment(userenvironment):
        APIOperationBase.__environment = userenvironment 
        return 
    
    def __init__(self, apiRequest):
        self._httpResponse = None
        self._request = None
        self._response = None
        #objectify variables 
        self._responseXML = None
        self._reponseObject = None
        self._mainObject = None
               
        if None == apiRequest:
            raise ValueError('Input request cannot be null')
         
        self._request = apiRequest
        __merchantauthentication = apicontractsv1.merchantAuthenticationType()
        APIOperationBase.__environment = constants.SANDBOX
        
        APIOperationBase.setmerchantauthentication(__merchantauthentication)

        if ( False == APIOperationBase.__classinitialized()):
            loggingfilename = utility.helper.getproperty(constants.propertiesloggingfilename)
            logginglevel = utility.helper.getproperty(constants.propertiesexecutionlogginglevel)
            
            if (None == loggingfilename):
                loggingfilename = constants.defaultLogFileName
            if (None == logginglevel):
                logginglevel = constants.defaultLoggingLevel
                
            logging.basicConfig(filename=loggingfilename, level=logginglevel, format=constants.defaultlogformat)
            __initialized = True

        self.validate()
            
        return
